﻿--{{{ Libraries

local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local L = AceLibrary("AceLocale-2.2"):new("GridStatusHOTStack")
local BS = AceLibrary("Babble-Spell-2.2")

--}}}
--
--{{{ Libraries

L:RegisterTranslations("enUS", function()
	return {
		["HOT Stack"] = true,

		["Color 1"] = true,
		["Color 2"] = true,
		["Color 3"] = true,
		["Color 4"] = true,
		["Color 5"] = true,
		["Color 6+"] = true,

		["Color when player has one heal-over-time buff"] = true,
		["Color when player has two heal-over-time buffs"] = true,
		["Color when player has three heal-over-time buffs"] = true,
		["Color when player has four heal-over-time buffs"] = true,
		["Color when player has five heal-over-time buffs"] = true,
		["Color when player has six or more heal-over-time buffs"] = true,
	}
end)

L:RegisterTranslations("koKR", function()
	return {
		["HOT Stack"] = "다량 강화 치유 지속",

		["Color 1"] = "색상 1",
		["Color 2"] = "색상 2",
		["Color 3"] = "색상 3",
		["Color 4"] = "색상 4",
		["Color 5"] = "색상 5",
		["Color 6+"] = "색상 6+",

		["Color when player has one heal-over-time buff"] = "플레이어에게 1개의 강화 치유 지속 버프 색상",
		["Color when player has two heal-over-time buffs"] = "플레이어에게 2개의 강화 치유 지속 버프 색상",
		["Color when player has three heal-over-time buffs"] = "플레이어에게 3개의 강화 치유 지속 버프 색상",
		["Color when player has four heal-over-time buffs"] = "플레이어에게 4개의 강화 치유 지속 버프 색상",
		["Color when player has five heal-over-time buffs"] = "플레이어에게 5개의 강화 치유 지속 버프 색상",
		["Color when player has six or more heal-over-time buffs"] = "플레이어에게 5개 이상의 강화 치유 지속 버프 색상",
	}
end)

--}}}

GridStatusHOTStack = GridStatus:NewModule("GridStatusHOTStack")
GridStatusHOTStack.menuName = "HOT Stack"
GridStatusHOTStack.options = false

--{{{ AceDB defaults

GridStatusHOTStack.defaultDB = {
	debug = false,
	alert_HOTStack = {
		text = "HOT Stack",
		enable = true,
		color = { r = 1, g = 0, b = 0, a = 1 },
		color2 = { r = 1, g = 1, b = 0, a = 1 },
		color3 = { r = 0, g = 1, b = 0, a = 1 },
		color4 = { r = 0, g = 1, b = 1, a = 1 },
		color5 = { r = 0, g = 0, b = 1, a = 1 },
		color6 = { r = 1, g = 0, b = 1, a = 1 },
		priority = 70,
		range = true,
	},
}

--}}}

--{{{ AceOptions table

local alert_HOTStackOptions = {
	["color2"] = {
		type = "color",
		name = L["Color 2"],
		desc = L["Color when player has two heal-over-time buffs"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color2
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color2
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["color3"] = {
		type = "color",
		name = L["Color 3"],
		desc = L["Color when player has three heal-over-time buffs"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color3
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color3
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["color4"] = {
		type = "color",
		name = L["Color 4"],
		desc = L["Color when player has four heal-over-time buffs"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color4
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color4
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["color5"] = {
		type = "color",
		name = L["Color 5"],
		desc = L["Color when player has five heal-over-time buffs"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color5
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color5
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
	["color6"] = {
		type = "color",
		name = L["Color 6+"],
		desc = L["Color when player has six or more heal-over-time buffs"],
		hasAlpha = true,
		get = function ()
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color6
			return color.r, color.g, color.b, color.a
		end,
		set = function (r, g, b, a)
			local color = GridStatusHOTStack.db.profile.alert_HOTStack.color6
			color.r = r
			color.g = g
			color.b = b
			color.a = a or 1
		end,
	},
}


function GridStatusHOTStack:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus('alert_HOTStack', "HOT Stack", alert_HOTStackOptions, true)
end


function GridStatusHOTStack:OnEnable()
	self:RegisterEvent("SpecialEvents_UnitBuffGained")
	self:RegisterEvent("SpecialEvents_UnitBuffLost")
	self:RegisterEvent("Grid_UnitJoined")
	self:RegisterEvent("Grid_UnitDeath")
end


function GridStatusHOTStack:Reset()
	self.super.Reset(self)
	self:UpdateAllUnits()
end


function GridStatusHOTStack:UpdateAllUnits()
	local name, status, statusTbl

	for name, status, statusTbl in self.core:CachedStatusIterator("alert_HOTStack") do
		self:Grid_UnitJoined(name)
	end
end


function GridStatusHOTStack:Grid_UnitJoined(name, unitid)
	if unitid then
		self:UpdateUnit(unitid)
	end
end


function GridStatusHOTStack:Grid_UnitDeath(unitname)
	local status, moduleName, desc
	for status, moduleName, desc in self.core:RegisteredStatusIterator() do
		if moduleName == self.name then
			self.core:SendStatusLost(unitname, status)
		end
	end
end


-- NOTE: Use Babble spell here below instead for localization

function GridStatusHOTStack:IsHOT( buff )
	if not buff then return false end
	return ( buff == BS["Renew"] ) or ( buff == BS["Rejuvenation"]) or ( buff == BS["Regrowth"] )
end


function GridStatusHOTStack:CountHOTs(unitid)
	local HOTcount = 0
	if unitid then
		for buffName, index, applications, texture, rank in Aura:BuffIter(unitid) do
			if self:IsHOT( buffName ) then
				HOTcount = HOTcount + 1
			end
		end
	end
	return HOTcount
end


function GridStatusHOTStack:SpecialEvents_UnitBuffGained(unit, buff, index, apps, tex, rank)
	self:Debug("UnitBuffGained:"..buff)
	if self:IsHOT( buff ) then
		self:UpdateUnit(unit, true)
	end
end


function GridStatusHOTStack:SpecialEvents_UnitBuffLost(unit, buff, index, apps, tex, rank)
	self:Debug("UnitBuffLost:"..buff)
	if self:IsHOT( buff ) then
		self:UpdateUnit(unit, true)
	end
end


function GridStatusHOTStack:UpdateUnit(unitid, ignoreRange)
	local name = UnitName(unitid)

	if not name then return end

	local settings = self.db.profile.alert_HOTStack
	local HOTCount = self:CountHOTs(unitid)

	self:Debug("Count:"..HOTCount)
	if HOTCount == 0 then
		self.core:SendStatusLost(name, "alert_HOTStack")
	else
		local thecolor = settings.color6
		if HOTCount == 1 then
			thecolor = settings.color
		elseif HOTCount == 2 then
			thecolor = settings.color2
		elseif HOTCount == 3 then
			thecolor = settings.color3
		elseif HOTCount == 4 then
			thecolor = settings.color4
		elseif HOTCount == 5 then
			thecolor = settings.color5
		end

		self.core:SendStatusGained(name, "alert_HOTStack",
			settings.priority,
			(settings.range and 40),
			thecolor,
			HOTCount,
			HOTCount,
			6 )
	end
end
